@@ -8,7 +8,6 @@ class OperationalError(Exception):
     """Raised on client operation errors."""
     pass
 
-class IndexedConnectionError(ConnectionError):
-    """Raised on socket errors that happen with operations that have opened index."""
+class RecoverableConnectionError(ConnectionError):
+    """Raised on socket connection errors that can be attempted to recover instantly."""
     pass
-  
\ No newline at end of file
