@@ -143,6 +143,8 @@ class Connection(object):
                 data = self.socket.recv(4096)
                 if self.debug:
                     print "DEBUG: read data bucket: %s" % data
+                if not data:
+                    raise RecoverableConnectionError('Connection closed on the remote end.')
             except socket.error, e:
                 self._die(e, 'Read error')
 
@@ -216,7 +218,7 @@ class HandlerSocket(threading.local):
         In case of connection failure on all available servers will raise
         :exc:`~.exceptions.ConnectionError`. If ``force_index`` is set, it will
         try only one connection that was used to open given ``index_id``. If that
-        fails will throw :exc:`~.exceptions.IndexedConnectionError`.
+        fails will throw :exc:`~.exceptions.RecoverableConnectionError`.
 
         :param index_id: index id to look up connection for, if ``None`` (default)
             or not found a new connection will be returned.
@@ -250,12 +252,9 @@ class HandlerSocket(threading.local):
                 # In case indexed connection is forced remove it from the caches
                 # and raise exception so higher level code could retry whole operation
                 if force_index:
-                    del self.index_map[index_id]
-                    for key, value in self.index_cache.items():
-                        if value == index_id:
-                            del self.index_cache[key]
+                    self.purge_index(index_id)
                     if connections:
-                        raise IndexedConnectionError('Could not use connection with given index id "%d"' % index_id)
+                        raise RecoverableConnectionError('Could not use connection with given index id "%d"' % index_id)
                     else:
                         # No point retrying if no more connections are available
                         raise exception(self.last_connection_exception)
@@ -364,6 +363,16 @@ class HandlerSocket(threading.local):
 
         self._clear_caches()
 
+    def purge_index(self, index_id):
+        """Clear single index connection and cache.
+
+        :param integer index_id: id of the index to purge.
+        """
+        del self.index_map[index_id]
+        for key, value in self.index_cache.items():
+            if value == index_id:
+                del self.index_cache[key]
+
     def _call(self, index_id, query, force_index=False):
         """Helper that performs actual data exchange with HandlerSocket server.
         Returns parsed response data.
@@ -376,8 +385,12 @@ class HandlerSocket(threading.local):
         :rtype: list
         """
         conn = self._get_connection(index_id, force_index)
-        conn.send('\t'.join(query)+'\n')
-        response = self._parse_response(conn.readline())
+        try:
+            conn.send('\t'.join(query)+'\n')
+            response = self._parse_response(conn.readline())
+        except ConnectionError, e:
+            self.purge_index(index_id)
+            raise e
 
         return response
 
