@@ -3,7 +3,7 @@ Should not be used externally.
 """
 from functools import wraps
 
-from exceptions import IndexedConnectionError
+from exceptions import RecoverableConnectionError
 
 
 def encode(value):
@@ -75,7 +75,7 @@ def retry_on_failure(func):
     def wrapper(*args, **kwargs):
         try:
             result = func(*args, **kwargs)
-        except IndexedConnectionError:
+        except RecoverableConnectionError:
             result = func(*args, **kwargs)
         return result
     return wrapper
